package com.io7m.example.ccatpm;

abstract class ArithmeticExpression
{
  /**
   * An integer constant.
   */

  public final static class ConstantExpression extends ArithmeticExpression
  {
    private final int value;

    @SuppressWarnings("synthetic-access") ConstantExpression(
      final int value)
    {
      super(ExpressionType.EXP_CONSTANT);
      this.value = value;
    }

    public final int getValue()
    {
      return this.value;
    }
  }

  static enum ExpressionType
  {
    EXP_CONSTANT,
    EXP_PLUS,
    EXP_MULTIPLY,
    EXP_SUBTRACT
  }

  /**
   * The product of two arithmetic expressions.
   */

  public final static class MultiplyExpression extends ArithmeticExpression
  {
    private final ArithmeticExpression e_left;
    private final ArithmeticExpression e_right;

    @SuppressWarnings("synthetic-access") MultiplyExpression(
      final ArithmeticExpression e_left,
      final ArithmeticExpression e_right)
    {
      super(ExpressionType.EXP_MULTIPLY);
      this.e_left = e_left;
      this.e_right = e_right;
    }

    public final ArithmeticExpression getLeft()
    {
      return this.e_left;
    }

    public final ArithmeticExpression getRight()
    {
      return this.e_right;
    }
  }

  /**
   * The sum of two arithmetic expressions.
   */

  public final static class PlusExpression extends ArithmeticExpression
  {
    private final ArithmeticExpression e_left;
    private final ArithmeticExpression e_right;

    @SuppressWarnings("synthetic-access") PlusExpression(
      final ArithmeticExpression e_left,
      final ArithmeticExpression e_right)
    {
      super(ExpressionType.EXP_PLUS);
      this.e_left = e_left;
      this.e_right = e_right;
    }

    public final ArithmeticExpression getLeft()
    {
      return this.e_left;
    }

    public final ArithmeticExpression getRight()
    {
      return this.e_right;
    }
  }

  /**
   * The difference of two arithmetic expressions.
   */

  public final static class SubtractExpression extends ArithmeticExpression
  {
    private final ArithmeticExpression e_left;
    private final ArithmeticExpression e_right;

    @SuppressWarnings("synthetic-access") SubtractExpression(
      final ArithmeticExpression e_left,
      final ArithmeticExpression e_right)
    {
      super(ExpressionType.EXP_SUBTRACT);
      this.e_left = e_left;
      this.e_right = e_right;
    }

    public final ArithmeticExpression getLeft()
    {
      return this.e_left;
    }

    public final ArithmeticExpression getRight()
    {
      return this.e_right;
    }
  }

  private final ExpressionType type;

  private ArithmeticExpression(
    final ExpressionType type)
  {
    this.type = type;
  }

  public final ExpressionType getType()
  {
    return this.type;
  }
}
